//
//  Toolkit.h
//  Toolkit
//
//  Created by Interface on 12/12/14.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

typedef unsigned long ULONG;
typedef unsigned int  UINT;

typedef int TkError;
enum {
    TKErrSuccess = 0,
    TKErrInvalidParameter = -2,
    TKErrNotInitialized = -1,
    TKErrInternal = -3,
    TKErrNoDevice = -4,
    TKErrNotSupported = -99,
    TKErrCancel = -100,
};

enum TKRepeatMode
{
    TKRepeatModeNone = 0,
    TKRepeatModeTitle = 1,
    TKRepeatModePlaylist = 2,
};

enum PlayItemInfoSelector
{
    PlayItemInfoSelectorSelected = 0,
    PlayItemInfoSelectorPlaying = 1,
};

enum PlaybackState
{
    PbStateStop = 0,
    PbStatePlayback = 1,
    PbStatePause = 2,
};

enum PlaylistState
{
    PlsAdded = 0,
    PlsDeleted,
    PlsModified,
    PlsCleared,
    PlsSwapped,
};

enum PlayItemSelectionState
{
    PissNone = 0,
    PissOne,
    PissPlural,
};

enum TkPlaylistStateChange
{
    TkPlaylistStateChangeTotalDuration = 0,
    TkPlaylistStateChangeReadonly = 1,
    TkPlaylistStateChangeModifiable = 2,
};

@protocol ITFAudioToolkitDelegate

- (void)songSwitch;
- (void)stateDidChange:(int)state;
- (void)notifyPos:(ULONG)pos_title total:(ULONG)pos_total;
- (void)playlistStateDidChange:(enum TkPlaylistStateChange)reason;
- (void)playlistSelectionDidChange:(bool)select;
- (void)propertiesMenuSelected;
- (void)deviceDidChange;

@end

#define LAST_SELECTED_DEVICE_NANE_KEY  (@"Last Selected Output Device Name")

#define RAM_MODE_SELECTION_INDEX_KEY   (@"RAM Mode Selection Index")
#define SELECTED_PLAY_MODE_NORMAL (0)
#define SELECTED_PLAY_MODE_RAM (1)

#define RAM_MODE_MEMORY_LIMITAITON_KEY (@"RAM Mode Memory Limitation")
#define RAM_MODE_MEMORY_LIMITATION_DEFAULT (100) // RAMモードのメモリ上限
#define RAM_MODE_MEMORY_LIMITATION_COEFF (1024 * 1024)

#define GAPLESS_THREASHOLD_DB_KEY (@"Gapless Threashold DB")

#define GAPLESS_ENABLE_KEY        (@"MP3 gapless play enable")
#define GAPLESS_THREASHOLD_DB_DEFAULT (-45.)
#define GAPLESS_THREASHOLD_DB_MAX (0.0)
#define GAPLESS_THREASHOLD_DB_MIN (-192.0)
#define GAPLESS_THRESHOLD_NONE    (0.) /**< ギャップレス処理OFF */

typedef NSMutableDictionary TkConfig;
@interface NSMutableDictionary (TkConfig)
@property (retain) NSString* deviceName;
@property (assign) int ramMode;
@property (assign) int ramModeMemoryLimit;
@property (assign) bool gaplessMode;
@property (assign) int gaplessCutoff;
@end


@interface TkPluginInfoItem : NSObject
{
    NSString* name;
    NSString* exts;
    NSString* vers;
}
- (id)initWithName:(NSString*)aName exts:(NSString*)aExt version:(NSString*)aVersion;
@property (readonly) NSString* pluginName;
@property (readonly) NSString* pluginExt;
@property (readonly) NSString* pluginVersion;
@end


@interface AudioFileInfo : NSObject
{
    NSString* fileName;
    NSString* titleName;
    NSString* artistName;
    NSString* extName;
    ULONG samplingRate;
    ULONG bitResolution;
    double durationBySec;
}
@property (retain) NSString* fileName;
@property (retain) NSString* titleName;
@property (retain) NSString* artistName;
@property (retain) NSString* extName;
@property (assign) ULONG samplingRate;
@property (assign) ULONG bitResolution;
@property (assign) double durationBySec;

@end

typedef struct _PlaylistSettingInfo
{
	/**
	 * @brief プレイリストの親となるビュー(NSView)のインスタンス
	 */
    void* parent;
    int left;
    int top;
    int width;
    int height;
    /**
     * @brief 各カラム要素の表示幅
     * [0]: Playing
     * [1]: Artist
     * [2]: Title
     * [3]: Duration
     */
    int column_width[4];
} PlaylistSettingInfo;

TkError Toolkit_Init(id <ITFAudioToolkitDelegate> delegate);
void Toolkit_Exit(void);
TkError Toolkit_OpenFile(void);
TkError Toolkit_OpenCD(void);
TkError Toolkit_AddFile(void);
TkError Toolkit_AddDroppedFiles(NSArray *paths);
TkError Toolkit_RemoveFile(void);
TkError Toolkit_NewPlaylist(void);
TkError Toolkit_LoadPlaylist(void);
TkError Toolkit_SavePlaylist(void);
TkError Toolkit_PlayRequest(void);
TkError Toolkit_StopRequest(void);
TkError Toolkit_PauseRequest(void);
TkError Toolkit_NextRequest(void);
TkError Toolkit_PrevRequest(void);
TkError Toolkit_Shuffle(bool onoff);
TkError Toolkit_Repeat(int mode);
TkError Toolkit_SetPos(UINT pos);
TkError Toolkit_GetPlaylist(PlaylistSettingInfo* info);
TkError Toolkit_SetPlaylist(PlaylistSettingInfo* info);
TkError Toolkit_GetFileInfo(bool select, AudioFileInfo * __autoreleasing *info);
TkError Toolkit_TotalTime(ULONG* time);
TkError Toolkit_GetDeviceList(NSArray* __autoreleasing *name);
TkError Toolkit_GetConfiguration(TkConfig *config);
TkError Toolkit_SetConfiguration(TkConfig *config);
TkError Toolkit_GetToolkitVersion(NSString* __autoreleasing * version);
TkError Toolkit_GetPluginVersion(NSArray* __autoreleasing * info);

TkError Toolkit_PrepareExit(void);

id getObject(void);
